@@ -20,32 +20,38 @@
 #include <dom/dom_string.h>
 #include <dom/css_stylesheet.h>
 #include <kdebug.h>
+#include <khtmlview.h>
 #include <kstandarddirs.h>
 #include <ktexteditor/editinterface.h>
 #include <ktexteditor/selectioninterface.h>
 #include <kstandarddirs.h>
 #include <klocale.h>
-//#include <kglobal.h>
-//#include <kcharsets.h>
+#include <kmultipledrag.h> 
+#include <kglobal.h>
+#include <kcharsets.h>
 
 #include <qregexp.h>
 #include <qfile.h>
 #include <qtextstream.h>
 #include <qdatetime.h>
+#include <qclipboard.h> 
 
 #include "document.h"
 #include "viewmanager.h"
-#include "quanta.h"
 #include "quantacommon.h"
 #include "resource.h"
 #include "undoredo.h"
 #include "node.h"
 #include "parser.h"
+#include "project.h"
 #include "tag.h"
 #include "nodeproperties.h"
 #include "htmlenhancer.h"
 #include "kafkacommon.h"
 #include "kafkaresource.h"
+#include "cursors.h"
+#include "kafkadragobject.h"
+#include "cursors.h"
 
 #include "wkafkapart.moc"
 
@@ -107,14 +113,14 @@ QString ab = i18n("Ident all");
 		file.close();
 	}
 
-	connect(m_kafkaPart, SIGNAL(domNodeInserted(DOM::Node, bool)),
-		this, SLOT(slotDomNodeInserted(DOM::Node, bool)));
-	connect(m_kafkaPart, SIGNAL(domNodeModified(DOM::Node)),
-		this, SLOT(slotDomNodeModified(DOM::Node)));
-	connect(m_kafkaPart, SIGNAL(domNodeIsAboutToBeRemoved(DOM::Node, bool)),
-		this, SLOT(slotDomNodeAboutToBeRemoved(DOM::Node, bool)));
-	connect(m_kafkaPart, SIGNAL(domNodeIsAboutToBeMoved(DOM::Node, DOM::Node, DOM::Node)),
-		this, SLOT(slotDomNodeIsAboutToBeMoved(DOM::Node, DOM::Node, DOM::Node)));
+	connect(m_kafkaPart, SIGNAL(domNodeInserted(DOM::Node, bool, NodeModifsSet*)),
+		this, SLOT(slotDomNodeInserted(DOM::Node, bool, NodeModifsSet*)));
+	connect(m_kafkaPart, SIGNAL(domNodeModified(DOM::Node, NodeModifsSet*)),
+		this, SLOT(slotDomNodeModified(DOM::Node, NodeModifsSet*)));
+	connect(m_kafkaPart, SIGNAL(domNodeIsAboutToBeRemoved(DOM::Node, bool, NodeModifsSet*)),
+		this, SLOT(slotDomNodeAboutToBeRemoved(DOM::Node, bool, NodeModifsSet*)));
+	connect(m_kafkaPart, SIGNAL(domNodeIsAboutToBeMoved(DOM::Node, DOM::Node, DOM::Node, NodeModifsSet*)),
+		this, SLOT(slotDomNodeIsAboutToBeMoved(DOM::Node, DOM::Node, DOM::Node, NodeModifsSet*)));
 
 	connect(m_kafkaPart, SIGNAL(domNodeNewCursorPos(DOM::Node, int)),
 		this, SLOT(slotdomNodeNewCursorPos(DOM::Node, int)));
@@ -148,7 +154,7 @@ void KafkaDocument::loadDocument(Document *doc)
 #endif
 	m_currentDoc = doc;
 
-	(static_cast<HTMLEnhancer *>(mainEnhancer))->setBaseURL(quantaApp->projectBaseURL());
+	(static_cast<HTMLEnhancer *>(mainEnhancer))->setBaseURL(Project::ref()->projectBaseURL());
 
 	//create a empty document with a basic tree : HTML, HEAD, BODY
 	m_kafkaPart->newDocument();
@@ -160,27 +166,27 @@ void KafkaDocument::loadDocument(Document *doc)
 	node = new Node(0L);
 	tag = new Tag();
 	tag->name = "#document";
-	tag->notInTree = true;
+	tag->setNotInTree(true);
 	node->tag = tag;
 	connectDomNodeToQuantaNode(m_kafkaPart->document(), node);
 	node = new Node(0L);
 	tag = new Tag();
 	tag->name = "HTML";
-	tag->notInTree = true;
+	tag->setNotInTree(true);
 	node->tag = tag;
 	connectDomNodeToQuantaNode(m_kafkaPart->document().firstChild(), node);
 	html = m_kafkaPart->document().firstChild();
 	node = new Node(0L);
 	tag = new Tag();
 	tag->name = "HEAD";
-	tag->notInTree = true;
+	tag->setNotInTree(true);
 	node->tag = tag;
 	connectDomNodeToQuantaNode(m_kafkaPart->document().firstChild().firstChild(), node);
 	head = m_kafkaPart->document().firstChild().firstChild();
 	node = new Node(0L);
 	tag = new Tag();
 	tag->name = "BODY";
-	tag->notInTree = true;
+	tag->setNotInTree(true);
 	node->tag = tag;
 	connectDomNodeToQuantaNode(m_kafkaPart->document().firstChild().lastChild(), node);
 	body = m_kafkaPart->document().firstChild().lastChild();
@@ -219,6 +225,7 @@ void KafkaDocument::loadDocument(Document *doc)
 	m_currentDoc->docUndoRedo->kafkaLoaded();
 	emit loaded();
 
+    m_currentDoc->docUndoRedo->syncKafkaCursorAndSelection(0);
 	//m_kafkaPart->document().updateRendering();
 
 #ifdef LIGHT_DEBUG
@@ -448,6 +455,38 @@ void KafkaDocument::disconnectAllDomNodes()
 	domNodeProps.clear();
 }
 
+void KafkaDocument::setCursorAndSelection(NodeSelectionInd *nodeSelection)
+{
+  bool cursorAtSelectionStart;
+  DOM::Node startDomNode, endDomNode, foo;
+  Node *startNode, *endNode;
+  int startOffset, endOffset;
+   
+  cursorAtSelectionStart = nodeSelection->cursorAtSelectionStart();
+  startNode = kafkaCommon::getNodeFromLocation(nodeSelection->cursorNode());
+  endNode = kafkaCommon::getNodeFromLocation(nodeSelection->cursorNodeEndSel());
+  
+  if(startNode && startNode->rootNode())
+    startDomNode = *(startNode->rootNode());
+  if(endNode && endNode->rootNode())
+    endDomNode = *(endNode->rootNode());
+  translateNodeIntoKafkaCursorPosition(startNode, nodeSelection->cursorOffset(), foo, (long&)startOffset);
+  translateNodeIntoKafkaCursorPosition(endNode, nodeSelection->cursorOffsetEndSel(), foo, (long&)endOffset);
+  
+  
+  if(cursorAtSelectionStart && !startDomNode.isNull())
+  {
+    m_kafkaPart->setCurrentNode(startDomNode, startOffset);
+  }
+  else if(!cursorAtSelectionStart && !endDomNode.isNull())
+  {
+    m_kafkaPart->setCurrentNode(endDomNode, endOffset);
+  }
+  
+  if(!startDomNode.isNull() && !endDomNode.isNull())
+    m_kafkaPart->setSelection(DOM::Range(startDomNode, (long)startOffset, endDomNode, (long)endOffset));
+}
+
 bool KafkaDocument::buildKafkaNodeFromNode(Node *node, bool insertNode)
 {
 #ifdef LIGHT_DEBUG
@@ -460,8 +499,17 @@ bool KafkaDocument::buildKafkaNodeFromNode(Node *node, bool insertNode)
 	Node *n, *parent;
 	int i;
 
-	if(node->tag->type == Tag::XmlTag || (node->tag->type == Tag::Text && !node->insideSpecial))
-	{
+//     This is a hack to not created DOM::Nodes from quanta empty nodes if outside body, because KHTML
+//     moves a node in that condition into the body and then the trees become desynchronized.
+    bool isInsideBody = false;
+    if(!m_currentDoc->defaultDTD()->name.contains("HTML", false))
+        isInsideBody = true;
+    else
+        isInsideBody = kafkaCommon::hasParent(node, "body");
+    
+    if(node->tag->type == Tag::XmlTag || 
+       ((node->tag->type == Tag::Text || (node->tag->type == Tag::Empty && isInsideBody)) && !node->insideSpecial))
+    {
 		str = node->tag->name.lower();
 
 		//The basics DOM::Nodes HTML, HEAD and BODY are loaded anyway, but we must now
@@ -492,8 +540,8 @@ bool KafkaDocument::buildKafkaNodeFromNode(Node *node, bool insertNode)
 			}
 			else
 			{
-				if(node->tag->type == Tag::Text)
-				{
+                if(node->tag->type == Tag::Text || node->tag->type == Tag::Empty)
+                {
 					newNode = kafkaCommon::createTextDomNode("", m_kafkaPart->document());
 				}
 				else
@@ -528,8 +576,8 @@ bool KafkaDocument::buildKafkaNodeFromNode(Node *node, bool insertNode)
 	/**	else if(node->parent->tag->str == "html")*/
 		else
 		{
-			if(node->tag->type == Tag::Text)
-			{
+            if(node->tag->type == Tag::Text || node->tag->type == Tag::Empty)
+            {
 				newNode = kafkaCommon::createTextDomNode("", m_kafkaPart->document());
 			}
 			else
@@ -548,8 +596,8 @@ bool KafkaDocument::buildKafkaNodeFromNode(Node *node, bool insertNode)
 		}
 
 		connectDomNodeToQuantaNode(newNode, node);
-		if(node->tag->type == Tag::Text)
-		{
+        if(node->tag->type == Tag::Text || node->tag->type == Tag::Empty)
+        {
 			//Now we get if the whitespaces at the left and right are kept or not.
 			//Left whitespaces are removed if:
 			//- It is the first child of a BLOCK
@@ -722,7 +770,8 @@ void KafkaDocument::buildNodeFromKafkaNode(Node *node, DOM::Node domNode)
 
 	if(!node) return;
 
-	node->tag->cleanStrBuilt = false;
+	node->tag->setCleanStrBuilt(false);
+        node->tag->setIndentationDone(false);
 	if(domNode.nodeType() == DOM::Node::TEXT_NODE)
 	{
 		node->tag->setStr(domNode.nodeValue().string());
@@ -762,7 +811,7 @@ Node * KafkaDocument::buildNodeFromKafkaNode(DOM::Node domNode, Node *nodeParent
 	}
 
 	//nodeParent can be the false body node which is not in the tree.
-	if(nodeParent->tag->notInTree)
+	if(nodeParent->tag->notInTree())
 		nodeParent = 0L;
 
 	/**_node = new Node(_nodeParent);*/
@@ -793,7 +842,10 @@ QString KafkaDocument::getDecodedChar(const QString &encodedChar)
 {
 	QMap<QString, QString>::Iterator it = encodedChars.find(encodedChar);
 	if(it == encodedChars.end())
-		return "";
+        {
+          //try this
+	  return KGlobal::charsets()->resolveEntities(encodedChar);;
+        }
 	return it.data();
 }
 
@@ -985,10 +1037,10 @@ QString KafkaDocument::generateCodeFromNode(Node *node, int bLine, int bCol, int
 		openingNode = node->getOpeningNode();
 		if(openingNode && openingNode->tag->type == Tag::ScriptTag)
 		{
-                  if(openingNode->tag->name.contains("XML PI") || 
-                    openingNode->tag->name.contains("PHP"))
+                  if(openingNode->tag->name.contains("XML PI", false) || 
+                    openingNode->tag->name.contains("PHP", false))
                     text = "?>";
-                  else if(openingNode->tag->name.contains("DTD"))
+                  else if(openingNode->tag->name.contains("DTD", false))
                     text = ">";
                   else
                     text = ">";
@@ -1015,22 +1067,22 @@ QString KafkaDocument::generateCodeFromNode(Node *node, int bLine, int bCol, int
 	else if(node->tag->type == Tag::ScriptTag)
 	{
 		//WARNING : HTML SPECIFIC
-		if(node->tag->name.contains("style"))
+        if(node->tag->name.contains("style", false))
 		{
 			text = "<" + QuantaCommon::tagCase("style") + ">";
 		}
-		else if(node->tag->name.contains("DTD"))
+		else if(node->tag->name.contains("DTD", false))
 		{
 			text = "<!";
 		}
-		else if(node->tag->name.contains("XML PI"))
+		else if(node->tag->name.contains("XML PI", false))
 		{
 			text = "<?xml";
 		}
-                else if(node->tag->name.contains("PHP"))
-                {
-                  text = "<?php";
-                }
+        else if(node->tag->name.contains("PHP", false))
+        {
+            text = "<?php";
+        }
 		bCol += text.length();
 		eCol = bCol - 1;
 		eLine = bLine;
@@ -1162,7 +1214,7 @@ void KafkaDocument::translateQuantaIntoNodeCursorPosition(uint line, uint col, N
   if(!*node)
    return;
   
-  if((*node)->tag->cleanStrBuilt)
+  if((*node)->tag->cleanStrBuilt() && (*node)->tag->indentationDone())
   {
     (*node)->tag->beginPos(beginLine, beginCol);
     curLine = beginLine;
@@ -1192,51 +1244,53 @@ void KafkaDocument::translateQuantaIntoNodeCursorPosition(uint line, uint col, N
 #endif
 }
 
-long KafkaDocument::translateKafkaIntoNodeCursorPosition(DOM::Node domNode, long domNodeOffset)
+void KafkaDocument::translateKafkaIntoNodeCursorPosition(DOM::Node domNode, long domNodeOffset, Node **node, int &offset)
 {
-	Node *node;
 	QString decodedText, encodedChar, encodedText, currentChar;
 	QChar curChar, oldChar;
 	long currentOffset;
 	bool waitForSpace = false, found = false;
-	int curNodeOffset, bLine, bCol, eLine, eCol;
+	int curNodeOffset, bLine = 0, bCol = 0, eLine, eCol;
 
+        offset = 0;
+        (*node) = 0L;
+        
 	if(domNode.isNull())
 	{
 #ifdef HEAVY_DEBUG
 		kdDebug(25001)<< "KafkaDocument::translateKafkaIntoNodeCursorPosition()" <<
 			" - DOM::Node not found!" << endl;
 #endif
-		return 0;
+		return ;
 	}
 
 	//get the corresponding Node*
-	node = getNode(domNode);
-	if(!node)
+	(*node) = getNode(domNode);
+	if(!(*node))
 	{
 #ifdef HEAVY_DEBUG
 		kdDebug(25001)<< "KafkaDocument::translateKafkaIntoNodeCursorPosition()" <<
 			" - Node not found!" << endl;
 #endif
-		return 0;
+		return ;
 	}
 
 	//If this node is selected (-1 means selected) then return "node selected" ;-)
 	if(domNodeOffset == -1)
 	{
-		return -1;
+		return ;
 	}
 
-	if(!node->tag->cleanStrBuilt)
+	if(!(*node)->tag->cleanStrBuilt())
 	{
 		//We NEED to have the up-to-date string in node.
-		node->tag->setStr(generateCodeFromNode(node, bLine, bCol, eLine, eCol));
+		(*node)->tag->setStr(generateCodeFromNode((*node), bLine, bCol, eLine, eCol));
 		//FIXME we shouldn't set it but if we don't the text will be re-encoded!
-		node->tag->cleanStrBuilt = true;
+		(*node)->tag->setCleanStrBuilt(true);
 	}
 
 	decodedText = domNode.nodeValue().string();
-	encodedText = node->tag->tagStr();
+	encodedText = (*node)->tag->tagStr();
 	currentOffset = domNodeOffset;
 	curNodeOffset = 0;
 
@@ -1245,7 +1299,7 @@ long KafkaDocument::translateKafkaIntoNodeCursorPosition(DOM::Node domNode, long
 		decodedText << ": encoded text:" << encodedText << ":" << endl;
 #endif
 
-	if(node->tag->type == Tag::Text)
+	if((*node)->tag->type == Tag::Text)
 	{
 		while(currentOffset > 0)
 		{
@@ -1297,7 +1351,7 @@ long KafkaDocument::translateKafkaIntoNodeCursorPosition(DOM::Node domNode, long
 	kdDebug(25001)<<"KafkaDocument::translateKafkaIntoNodeCursorPosition() - " <<
 		curNodeOffset << endl;
 #endif
-	return curNodeOffset;
+	offset = curNodeOffset;
 }
 
 void KafkaDocument::translateKafkaIntoQuantaCursorPosition(DOM::Node _currentDomNode, int offset, int &line, int &col)
@@ -1384,7 +1438,8 @@ void KafkaDocument::translateKafkaIntoQuantaCursorPosition(DOM::Node _currentDom
 
 	if(offset == -1)
 	{
-		m_currentDoc->selectionIf->setSelection(curLine, curCol, endLine, endCol + 1);
+    if (m_currentDoc->selectionIf)
+		  m_currentDoc->selectionIf->setSelection(curLine, curCol, endLine, endCol + 1);
 		line = endLine;
 		col = endCol + 1;
 		return;
@@ -1435,7 +1490,7 @@ void KafkaDocument::translateKafkaIntoQuantaCursorPosition(DOM::Node _currentDom
 				}
 				else if((curCol + encodedChar.length()) >= currentLine.length())
 				{
-					curLine++;
+					++curLine;
 					curCol = 0;
 					currentLine = m_currentDoc->editIf->textLine(curLine);
 					if(curLine < endLine)
@@ -1447,7 +1502,7 @@ void KafkaDocument::translateKafkaIntoQuantaCursorPosition(DOM::Node _currentDom
 				}
 			}
 			oldChar = curChar;
-			currentOffset--;
+			--currentOffset;
 		}
 	}
 	else if(_currentNode->tag->type == Tag::XmlTag)
@@ -1464,12 +1519,12 @@ void KafkaDocument::translateKafkaIntoQuantaCursorPosition(DOM::Node _currentDom
 					closingNode->tag->endPos(curLine, curCol);
 				else
 					_currentNode->tag->endPos(curLine, curCol);
-				curCol++;
+				++curCol;
 			}
 			else
 			{
 				_currentNode->tag->endPos(curLine, curCol);
-				curCol++;
+				++curCol;
 			}
 		}
 		else
@@ -1495,7 +1550,7 @@ void KafkaDocument::translateKafkaIntoQuantaCursorPosition(DOM::Node _currentDom
 				while(i < offset && node && node->SNext())
 				{
 					node = node->SNext();
-					i++;
+					++i;
 				}
 				if(!node)
 					node = _currentNode;
@@ -1508,7 +1563,7 @@ void KafkaDocument::translateKafkaIntoQuantaCursorPosition(DOM::Node _currentDom
 						closingNode->tag->endPos(curLine, curCol);
 					else
 						node->tag->endPos(curLine, curCol);
-					curCol++;
+					++curCol;
 				}
 			}
 		}
@@ -1529,7 +1584,7 @@ void KafkaDocument::translateNodeIntoKafkaCursorPosition(Node *node, int offset,
 	int col;
 	bool lookForEntity, lookForSpaces, found;
 
-	if(node->rootNode() && node->rootNode()->nodeType() == DOM::Node::TEXT_NODE)
+	if(node && node->rootNode() && node->rootNode()->nodeType() == DOM::Node::TEXT_NODE)
 	{
 		domNodeOffset = 0;
 		domNode = *node->rootNode();
@@ -1545,7 +1600,7 @@ void KafkaDocument::translateNodeIntoKafkaCursorPosition(Node *node, int offset,
 			else if(curChar[0].isSpace())
 				lookForSpaces = true;
 			found = false;
-			while(!found)
+			while(!found && col < offset)
 			{
 				if((lookForEntity && curChar == ";") ||
 					!(lookForSpaces || lookForEntity))
@@ -1561,7 +1616,7 @@ void KafkaDocument::translateNodeIntoKafkaCursorPosition(Node *node, int offset,
 			domNodeOffset++;
 		}
 	}
-	else if(node->rootNode())
+	else if(node && node->rootNode())
 	{
 		domNode = *node->rootNode();
 		domNodeOffset = 0;//shoud we select?
@@ -1749,7 +1804,7 @@ void KafkaDocument::coutLinkTree(Node *, int)
 #endif
 }
 
-void KafkaDocument::slotDomNodeInserted(DOM::Node domNode, bool insertChilds)
+void KafkaDocument::slotDomNodeInserted(DOM::Node domNode, bool insertChilds, NodeModifsSet* modifs)
 {
 #ifdef LIGHT_DEBUG
 	if(!domNode.isNull())
@@ -1761,7 +1816,6 @@ void KafkaDocument::slotDomNodeInserted(DOM::Node domNode, bool insertChilds)
 	Node *_nodeParent = 0L, *nodeNext = 0L, *_node = 0L;
 	DOM::Node tmpDomNode, nextDomNode;
 	bool b = false;
-	NodeModifsSet *modifs;
 
 #ifdef LIGHT_DEBUG
 	QTime t;
@@ -1789,7 +1843,6 @@ void KafkaDocument::slotDomNodeInserted(DOM::Node domNode, bool insertChilds)
 		}
 	}
 
-	modifs = new NodeModifsSet();
 	_node = buildNodeFromKafkaNode(domNode, _nodeParent, nodeNext, 0, 0L, 0, modifs);
 
 	if(insertChilds && domNode.hasChildNodes())
@@ -1803,7 +1856,6 @@ void KafkaDocument::slotDomNodeInserted(DOM::Node domNode, bool insertChilds)
 			tmpDomNode = kafkaCommon::getNextDomNode(tmpDomNode, b, false, domNode);
 		}
 	}
-	m_currentDoc->docUndoRedo->addNewModifsSet(modifs, undoRedo::KafkaModif);
 
 #ifdef LIGHT_DEBUG
 	kdDebug(25001)<< "KafkaDocument::slotDomNodeInserted() in " << t.elapsed() <<
@@ -1815,7 +1867,7 @@ void KafkaDocument::slotDomNodeInserted(DOM::Node domNode, bool insertChilds)
 
 }
 
-void KafkaDocument::slotDomNodeModified(DOM::Node domNode)
+void KafkaDocument::slotDomNodeModified(DOM::Node domNode, NodeModifsSet* modifs)
 {
 #ifdef LIGHT_DEBUG
 	if(!domNode.isNull())
@@ -1825,7 +1877,6 @@ void KafkaDocument::slotDomNodeModified(DOM::Node domNode)
 		kdDebug(25001)<< "KafkaDocument::slotDomNodeModfied() - DOM::Node: NULL" << endl;
 #endif
 	Node *node = 0L;
-	NodeModifsSet *modifs;
 	NodeModif *modif;
 	kNodeAttrs *props, *newProps;
 	DOM::Node newDomNode, parentDomNode, nextSiblingDomNode;
@@ -1867,9 +1918,7 @@ void KafkaDocument::slotDomNodeModified(DOM::Node domNode)
 
 		buildNodeFromKafkaNode(node, domNode);
 
-		modifs = new NodeModifsSet();
 		modifs->addNodeModif(modif);
-		m_currentDoc->docUndoRedo->addNewModifsSet(modifs, undoRedo::KafkaModif);
 	}
 	else
 	{
@@ -1922,6 +1971,13 @@ void KafkaDocument::slotDomNodeModified(DOM::Node domNode)
 				getNode(nextSiblingDomNode), modifs, false);
 			props->setNode(node);
 			modifs->addNodeModif(modif);
+                        //If there is a empty Node after of before the text, remove them
+                        if(node->prev && node->prev->tag->type == Tag::Empty)
+                          kafkaCommon::extractNode(node->prev, modifs);
+                        if(node->next && node->next->tag->type == Tag::Empty)
+                          kafkaCommon::extractNode(node->next, modifs);
+                          
+                        //Log the changes
 			m_currentDoc->docUndoRedo->addNewModifsSet(modifs, undoRedo::KafkaModif);
 		}
 
@@ -1940,7 +1996,7 @@ void KafkaDocument::slotDomNodeModified(DOM::Node domNode)
 #endif
 }
 
-void KafkaDocument::slotDomNodeAboutToBeRemoved(DOM::Node _domNode, bool deleteChilds)
+void KafkaDocument::slotDomNodeAboutToBeRemoved(DOM::Node _domNode, bool deleteChilds, NodeModifsSet* modifs)
 {
 #ifdef LIGHT_DEBUG
 	if(!_domNode.isNull())
@@ -1954,7 +2010,6 @@ void KafkaDocument::slotDomNodeAboutToBeRemoved(DOM::Node _domNode, bool deleteC
 	Node *_node = 0L, *_nodeNext = 0L, *_tmpNode = 0L, *n = 0L;
 	int i, bLine, bCol, eLine, eCol, bLine2, bCol2;
 	bool hasClosingNode = false, b;
-	NodeModifsSet *modifs;
 	NodeModif *modif;
 
 #ifdef LIGHT_DEBUG
@@ -1970,7 +2025,6 @@ void KafkaDocument::slotDomNodeAboutToBeRemoved(DOM::Node _domNode, bool deleteC
 		return;
 	}
 
-	modifs = new NodeModifsSet();
 
 	//If we are deleting a PHP Node which is embedded into a tag e.g. <a <? echo boo; ?> >
 	//We must regenerate the <a> tag string.
@@ -1980,7 +2034,7 @@ void KafkaDocument::slotDomNodeAboutToBeRemoved(DOM::Node _domNode, bool deleteC
 		_node->parent->tag->endPos(eLine, eCol);
 		_node->tag->beginPos(bLine2, bCol2);
 		if(QuantaCommon::isBetween(bLine2, bCol2, bLine, bCol, eLine,eCol) == 0)
-			_node->parent->tag->cleanStrBuilt = false;
+			_node->parent->tag->setCleanStrBuilt(false);
 	}
 
 	if(_node->prev)
@@ -2130,7 +2184,6 @@ void KafkaDocument::slotDomNodeAboutToBeRemoved(DOM::Node _domNode, bool deleteC
 		modifs->addNodeModif(modif);
 		_node = _nodeNext;
 	}
-	m_currentDoc->docUndoRedo->addNewModifsSet(modifs, undoRedo::KafkaModif);
 
 	//NO NORMALIZATION!! It is KafkaWidget::normalize()'s job!
 
@@ -2144,9 +2197,8 @@ void KafkaDocument::slotDomNodeAboutToBeRemoved(DOM::Node _domNode, bool deleteC
 
 }
 
-void KafkaDocument::slotDomNodeIsAboutToBeMoved(DOM::Node domNode, DOM::Node newParent, DOM::Node before)
+void KafkaDocument::slotDomNodeIsAboutToBeMoved(DOM::Node domNode, DOM::Node newParent, DOM::Node before, NodeModifsSet* modifs)
 {
-	NodeModifsSet *modifs;
 	Node *node, *parent, *nextSibling, *closingNode;
 
 	if(domNode.isNull())
@@ -2161,13 +2213,11 @@ void KafkaDocument::slotDomNodeIsAboutToBeMoved(DOM::Node domNode, DOM::Node new
 
 	closingNode = node->getClosingNode();
 
-	modifs = new NodeModifsSet();
 	kafkaCommon::moveNode(node, parent, nextSibling, modifs, false);
 
 	if(closingNode)
 		kafkaCommon::moveNode(closingNode, parent, nextSibling, modifs, false);
 
-	m_currentDoc->docUndoRedo->addNewModifsSet(modifs, undoRedo::KafkaModif);
 }
 
 void KafkaDocument::slotdomNodeNewCursorPos(DOM::Node, int)
@@ -2180,3 +2230,120 @@ void KafkaDocument::slotdomNodeNewCursorPos(DOM::Node, int)
 	//getQuantaCursorPosition(line, col);
 	//emit newCursorPosition(line, col);
 }
+
+void KafkaDocument::slotCut()
+{
+    QString text = m_kafkaPart->selectedText();
+        
+    NodeSelectionInd selection_ind;
+    selection_ind.fillWithVPLCursorSelection();
+    
+    int startOffset = selection_ind.cursorOffset();
+    int endOffset = selection_ind.cursorOffsetEndSel();
+    Node* startNode = kafkaCommon::getNodeFromLocation(selection_ind.cursorNode());
+    Node* endNode = kafkaCommon::getNodeFromLocation(selection_ind.cursorNodeEndSel());
+    
+    DOM::Node cursorDomNode;
+    int cursorOffset;
+    
+    m_kafkaPart->getCurrentNode(cursorDomNode, cursorOffset);
+    Node* cursorNode = getNode(cursorDomNode);
+    
+    slotCut(startNode, startOffset, endNode, endOffset, &cursorNode, cursorOffset, text);
+}
+
+void KafkaDocument::slotCut(Node* startNode, int startOffset, Node* endNode, int endOffset, 
+                            Node** cursorNode, int cursorOffset, QString const& text)
+{
+    if(!startNode || !endNode)
+        return;
+    
+    NodeModifsSet *modifs = new NodeModifsSet();
+    
+    Node* subtree_root = kafkaCommon::DTDExtractNodeSubtree(startNode, startOffset, endNode, endOffset, 
+            cursorNode, cursorOffset, modifs);
+    
+    m_currentDoc->docUndoRedo->addNewModifsSet(modifs, undoRedo::NodeTreeModif);
+    
+    if(subtree_root)
+    {        
+        KafkaDragObject* node_drag = new KafkaDragObject(subtree_root);
+        QTextDrag* text_drag = new QTextDrag(text);
+        KMultipleDrag* drag_object = new KMultipleDrag();
+        drag_object->addDragObject(node_drag);
+        drag_object->addDragObject(text_drag);
+        
+        QApplication::clipboard()->setData(drag_object);
+#ifdef LIGHT_DEBUG
+        kafkaCommon::coutTree(subtree_root, 3);
+#endif
+    }   
+}
+
+void KafkaDocument::slotCopy()
+{
+    QString text = m_kafkaPart->selectedText();
+        
+    NodeSelectionInd selection_ind;
+    selection_ind.fillWithVPLCursorSelection();
+    
+    int startOffset = selection_ind.cursorOffset();
+    int endOffset = selection_ind.cursorOffsetEndSel();
+    Node* startNode = kafkaCommon::getNodeFromLocation(selection_ind.cursorNode());
+    Node* endNode = kafkaCommon::getNodeFromLocation(selection_ind.cursorNodeEndSel());
+
+    slotCopy(startNode, startOffset, endNode, endOffset, text);
+}
+
+void KafkaDocument::slotCopy(Node* startNode, int startOffset, Node* endNode, int endOffset, QString const& text)
+{
+    if(!startNode || !endNode)
+        return;
+    
+    Node* subtree_root = kafkaCommon::getNodeSubtree(startNode, startOffset, endNode, endOffset);
+    
+    if(subtree_root)
+    {
+        KafkaDragObject* node_drag = new KafkaDragObject(subtree_root);
+        QTextDrag* text_drag = new QTextDrag(text);
+        KMultipleDrag* drag_object = new KMultipleDrag();
+        drag_object->addDragObject(node_drag);
+        drag_object->addDragObject(text_drag);
+        
+        QApplication::clipboard()->setData(drag_object);
+        // FIXME delete the subtree 
+#ifdef LIGHT_DEBUG
+        kafkaCommon::coutTree(subtree_root, 3);
+#endif
+    }    
+}
+
+void KafkaDocument::slotPaste()
+{
+    QClipboard *cb = QApplication::clipboard();
+    QMimeSource* e = cb->data();
+    Node* node = new Node(0);
+    
+    if(KafkaDragObject::decode(e, node))
+    {
+        bool go_up = false;
+        for(Node* aux = node; aux; aux = kafkaCommon::getNextNode(aux, go_up))
+            kafkaCommon::restorePastedNode(aux, getCurrentDoc());
+        
+        NodeSelectionInd selection_ind;
+        selection_ind.fillWithVPLCursorSelection();
+    
+        Node* cursorNode = kafkaCommon::getNodeFromLocation(selection_ind.cursorNode());
+        int cursorOffset = selection_ind.cursorOffset();
+        
+        NodeModifsSet *modifs = new NodeModifsSet();
+        
+        if(selection_ind.hasSelection())
+            kafkaCommon::DTDRemoveSelection(selection_ind, &cursorNode, cursorOffset, modifs);
+        else
+            cursorNode = 0; // use selection_ind
+        kafkaCommon::DTDInsertNodeSubtree(node, selection_ind, &cursorNode, cursorOffset, modifs);
+        
+        m_currentDoc->docUndoRedo->addNewModifsSet(modifs, undoRedo::NodeTreeModif, 0, false);
+    }
+}
