@@ -61,9 +61,10 @@
 #include "qextfileinfo.h"
 #include "resource.h"
 
-ProjectUpload::ProjectUpload(const KURL& url, bool showOnlyProfiles, const char* name)
-  :ProjectUploadS( 0L, name, true, Qt::WDestructiveClose)
+ProjectUpload::ProjectUpload(const KURL& url, bool showOnlyProfiles, bool quickUpload, const char* name)
+  :ProjectUploadS( 0L, name, false, Qt::WDestructiveClose)
 {
+    m_quickUpload = quickUpload;
     m_profilesOnly = showOnlyProfiles;
     list->hide();
     m_project = Project::ref();
@@ -183,15 +184,15 @@ void ProjectUpload::slotBuildTree()
      UploadTreeFile *it = list->addItem(u, item);
      if ( it != 0 )
      {
-       int uploadedTime;
+       int uploadedTime = -1;
        if (m_uploadTimeList.contains(s))
          uploadedTime = m_uploadTimeList[s];
        int modifiedTime = item.time(KIO::UDS_MODIFICATION_TIME);
        el.setAttribute("modified_time", modifiedTime);
        int uploadStatus = el.attribute("uploadstatus", "1").toInt();
-       if (forceUpload && uploadStatus == 0)
+       if (m_quickUpload || (forceUpload && uploadStatus == 0))
         uploadStatus = 1;
-       if (uploadedTime != modifiedTime && uploadStatus != 0)
+       if (m_quickUpload || (uploadedTime != modifiedTime && uploadStatus != 0))
        {
          modified.append( u );
          it->setSelected(true);
@@ -215,6 +216,8 @@ void ProjectUpload::slotBuildTree()
  //hack to force repainting of the treeview
  resize(width() + 1, height());
  resize(width() - 1, height());
+ if (m_quickUpload)
+   startUpload();
 }
 
 
@@ -356,14 +359,15 @@ void ProjectUpload::startUpload()
       return;
     } else
     {
-      if (KMessageBox::warningYesNo(this, i18n("<qt><b>%1</b> seems to be unaccessible.<br>Do you want to proceed with upload?</qt>")
-                                          .arg(u.prettyURL(0, KURL::StripFileProtocol))) == KMessageBox::Yes)
+      if (KMessageBox::warningContinueCancel(this, i18n("<qt><b>%1</b> seems to be unaccessible.<br>Do you want to proceed with upload?</qt>")
+          .arg(u.prettyURL(0, KURL::StripFileProtocol)),QString::null,KStdGuiItem::cont()) == KMessageBox::Continue)
       {
         upload();
         return;
       }
     }
   }
+  buttonUpload->setEnabled(true);
   uploadInProgress = false;
 }
 
@@ -415,6 +419,8 @@ void ProjectUpload::upload()
         if (!QExtFileInfo::createDir(dir))
         {
           QuantaCommon::dirCreationError(this, KURL( dir.prettyURL(0, KURL::StripFileProtocol) ));
+          buttonUpload->setEnabled(true);
+          uploadInProgress = false;
           return;
         }
       }
@@ -442,6 +448,7 @@ void ProjectUpload::upload()
       }
   }
   saveRemoteUploadInfo();
+  buttonUpload->setEnabled(true);
   uploadInProgress = false;
   reject();
 }
@@ -454,6 +461,7 @@ void ProjectUpload::uploadFinished( KIO::Job *job )
    {
       job->showErrorDialog( this  );
       uploadInProgress = false;
+      buttonUpload->setEnabled(true);
       return;
    }
    KIO::FileCopyJob *fJob = dynamic_cast<KIO::FileCopyJob *>(job);
@@ -524,7 +532,7 @@ void ProjectUpload::slotUploadNext()
   {
     totalProgress->setProgress(totalProgress->progress()+1);
    // QListViewItem *it = list->findItem( currentURL.path() );
-   QListViewItem *it = currentItem;
+    QListViewItem *it = currentItem;
     if (it)
     {
      it->setSelected(false);
